"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Retrier = void 0;
class Retrier {
    constructor(_ErrorEquals, _IntervalSeconds = 1, _MaxAttempts = 3, _BackoffRate = 2.0) {
        this._ErrorEquals = _ErrorEquals;
        this._IntervalSeconds = _IntervalSeconds;
        this._MaxAttempts = _MaxAttempts;
        this._BackoffRate = _BackoffRate;
        this.currentNumberOfRetries = 0;
        this.currentIntervalSeconds = _IntervalSeconds;
    }
    static create(taskRetryRule) {
        return new Retrier(taskRetryRule.ErrorEquals, taskRetryRule.IntervalSeconds, taskRetryRule.MaxAttempts, taskRetryRule.BackoffRate);
    }
    async retry(fn) {
        let output;
        try {
            output = await fn();
            return output;
        }
        catch (error) {
            if (this.currentNumberOfRetries < this._MaxAttempts) {
                this.currentNumberOfRetries++;
                return await new Promise((resolve, reject) => {
                    setTimeout(async () => {
                        this.currentIntervalSeconds = this.currentIntervalSeconds * this._BackoffRate;
                        try {
                            const output = await this.retry(fn);
                            return resolve(output);
                        }
                        catch (error) {
                            return reject(error);
                        }
                    }, this.currentIntervalSeconds * 1000);
                });
            }
            else {
                throw error;
            }
        }
    }
    get ErrorEquals() {
        return this._ErrorEquals;
    }
}
exports.Retrier = Retrier;
