"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Retriers = void 0;
const Retrier_1 = require("./Retrier");
const StatesErrors_1 = require("./StatesErrors");
class Retriers {
    constructor(_retriers) {
        this._retriers = _retriers;
    }
    static create(taskRetryRules) {
        const retriers = taskRetryRules.map((taskRetryRule) => {
            return Retrier_1.Retrier.create(taskRetryRule);
        });
        return new Retriers(retriers);
    }
    getRetrierBasedOn(statesError) {
        const retrier = this._retriers.find((retrier) => {
            return retrier.ErrorEquals.includes(statesError);
        });
        if (!retrier) {
            return;
        }
        return retrier;
    }
    async retry(fn) {
        const retrier = this.getRetrierBasedOn(StatesErrors_1.StatesErrors.TaskFailed);
        if (retrier) {
            return await retrier.retry(fn);
        }
        else {
            return fn();
        }
    }
}
exports.Retriers = Retriers;
